import Foundation
import SQLite
import HealthKit
import HealthDB

extension Database {

    private static let databaseFileUrl = Bundle.main.url(forResource: "healthdb_secure", withExtension: "sqlite")

    static var mock: Database {
        let bundleUrl = Database.databaseFileUrl!
        let local = FileManager.default.documentDirectory.appendingPathComponent("db.sqlite")
        if !FileManager.default.fileExists(atPath: local.path) {
            try! FileManager.default.copyItem(at: bundleUrl, to: local)
        }
        let store = try! HealthDatabase(fileUrl: local)
        return .init(store: store)
    }

    static var empty: Database {

        do {
            let connection = try Connection(.inMemory)
            let store =  HKDatabaseStore(database: connection)
            try store.createTables()
            try store.insert(workout: .mock1)
            return .init(store: .init(wrapping: store))
        } catch {
            print(error)
            fatalError("Failed to create empty database: \(error)")
        }
    }
}
